from mongoengine import *


class PropertyAddressModel(DynamicEmbeddedDocument):
    state = StringField()
    postalCode = StringField()
    cityName = StringField()
    regionCode = StringField()
    countryCode = StringField()
    addressLine = ListField(StringField())


class PropertyModel(DynamicDocument):
    meta = {'collection': 'properties'}

    _id = StringField(primary_key=True)
    name = StringField()
    orgId = StringField()
    clientId = StringField()
    accommodationDetails = DictField()
    address = EmbeddedDocumentField(PropertyAddressModel)
    alternateHotels = ListField(DictField())
    buildings = ListField(DictField())
    communication = DictField()
    generalInformation = DictField()
    hotelCorporateInformations = DictField()
    hotelRateRanges = ListField(DictField())
    primaryDetails = DictField()
    propertyControls = DictField()
    roomClasses = ListField(DictField())
    roomHierarchies = ListField(DictField())
    rooms = ListField(DictField())
    airports = ListField(DictField())
    transportations = ListField(DictField())
    creditCardTypes = ListField(DictField())
    hotelAmenities = ListField(DictField())
    attractions = ListField(DictField())
    accountsReceivableDetails = ListField(DictField())
    taxType = ListField(DictField())
    leisureManagement = DictField()
    transactionCodes = ListField(DictField())
    cateringMenu = DictField()
    eventFunctionSpaces = ListField(DictField())
    eventForecasts = ListField(DictField())
    eventInventoryItems = ListField(DictField())
    eventCodes = ListField(DictField())
    activityConfigDetails = DictField()
    eventRevenueTypes = DictField()
    ratePlan = DictField()
    

if __name__ == '__main__':
    import upswingutil as ul
    ul.MONGO_URI = 'mongodb://AdminUpSwingGlobal:Upswing098812Admin0165r@dev.db.upswing.global:27017/?authSource=admin&readPreference=primary&appname=Agent%20Oracle%20Dev&ssl=false'
    from upswingutil.db import MongodbV2
    mongo = MongodbV2('OHIPSB2')
    prop = PropertyModel(_id='SAND02')
    prop.orgId = 'OHIPSB2'
    prop.clientId = 'SAND020'
    prop.name = "SAmple"
    address = PropertyAddressModel()
    address.state = 'test'
    address.addressLine = ['sa', 'sdfs', 'test']
    address.cityName = 'temt2'
    prop.address = address
    prop.primaryDetails = {
        'legal': 'smwekmw'
    }
    mongo.save(prop)
    mongo.close_connection()



from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry
import requests


def http_retry():
    retry_strategy = Retry(
        total=3,
        status_forcelist=[400, 403, 429, 500, 502, 503, 504],
        method_whitelist=["HEAD", "POST", "GET", "OPTIONS"]
    )
    adapter = HTTPAdapter(max_retries=retry_strategy)
    http = requests.Session()
    http.mount("https://", adapter)
    http.mount("http://", adapter)
    return http



import unittest
from upswingutil.ml import ReservationCountForecast
import upswingutil as ul
from upswingutil.pms.oracle import NAME as ORACLE
from upswingutil.pms.rms import NAME as RMS
from upswingutil.db import Mongodb

ul.ENCRYPTION_SECRET = "S1335HwpKYqEk9CM0I2hFX3oXa5T2oU86OXgMSW4s6U="
ul.MONGO_URI = "mongodb://AdminUpSwingGlobal:Upswing098812Admin0165r@dev.db.upswing.global:27017/?authSource=admin&readPreference=primary&appname=Agent%20Oracle%20Dev&ssl=false"
ul.G_CLOUD_PROJECT = "aura-staging-31cae"
ul.FIREBASE = "/Users/harsh/upswing/github/api-oracle/SECRET/aura-staging-31cae-firebase-adminsdk-dyolr-7c135838e9.json"
ul.LOG_LEVEL_VALUE = 'DEBUG'


class TestMongodb(unittest.TestCase):

    def test_connecting_to_mongodb(self):
        mongo = Mongodb('OHIPSB')
        result = mongo.get_collection(mongo.RESERVATION_COLLECTION).count()
        print(f'This collection has {result} records')
        mongo.close_connection()

    def test_query_mongodb(self):
        mongo = Mongodb('11249')
        result = mongo.get_collection(mongo.RESERVATION_COLLECTION).find({'_id': 6},
                                                                         {'accountId': 1, 'activityId': 1, 'areaId': 1,
                                                                          'booking_type': 1})
        for item in result:
            print(item)
        mongo.close_connection()

    def test_aggregate_pipeline(self):
        mongo = Mongodb('11249')
        propertyId = 11264
        __pipeline__ = [
            {
                '$match': {
                    'clientId': propertyId
                }
            }, {
                '$group': {
                    '_id': '$booking_type',
                    'count': {
                        '$sum': 1
                    }
                }
            }
        ]
        result = mongo.execute_pipeline(mongo.RESERVATION_COLLECTION, __pipeline__)
        for item in result:
            print(f'{item["_id"]} has {item["count"]} entries')
        mongo.close_connection()



import unittest
from upswingutil.ml import ReservationCountForecast
import upswingutil as ul
from upswingutil.pms.oracle import NAME as ORACLE
from upswingutil.pms.rms import NAME as RMS

ul.ENCRYPTION_SECRET = "S1335HwpKYqEk9CM0I2hFX3oXa5T2oU86OXgMSW4s6U="
ul.MONGO_URI = "mongodb://AdminUpSwingGlobal:Upswing098812Admin0165r@dev.db.upswing.global:27017/?authSource=admin&readPreference=primary&appname=Agent%20Oracle%20Dev&ssl=false"
ul.G_CLOUD_PROJECT = "aura-staging-31cae"
ul.FIREBASE = "/Users/harsh/upswing/github/api-oracle/SECRET/aura-staging-31cae-firebase-adminsdk-dyolr-7c135838e9.json"
ul.LOG_LEVEL_VALUE = 'DEBUG'


class TestReservationCountForecasting(unittest.TestCase):

    def test_creating_model_oracle(self):
        """
        Training model for ORACLE
        """
        orgId = 'OHIPSB'
        propertyId = 'SAND01'
        resvCountForecast = ReservationCountForecast(ORACLE, orgId, propertyId)
        resvCountForecast.train()

    def test_creating_model_rms(self):
        """
        Training model for RMS
        """
        orgId = '11249'
        propertyId = '11264'
        resvCountForecast = ReservationCountForecast(RMS, orgId, propertyId)
        resvCountForecast.train()

    def test_predict_rms(self):
        orgId = '11249'
        propertyId = '11264'
        rcf = ReservationCountForecast(RMS, orgId, propertyId)
        startDate = '2021-01-01'
        endDate = '2021-02-01'
        key, val = rcf.predict(startDate, endDate)
        avg = (sum(val) / len(val))
        self.assertTrue(len(key) > 0)
        self.assertTrue(len(val) > 0)
        self.assertTrue(avg >= 0)
        print(f'Avg predicted value for {startDate} till {endDate} is {avg}')


if __name__ == '__main__':
    unittest.main()



from upswingutil.pms.oracle import ReservationSync
import firebase_admin
from loguru import logger
import os
from upswingutil.db import MongodbV2
from upswingutil.db.model import GuestsModel

import upswingutil as ul
ul.ENCRYPTION_SECRET = "S1335HwpKYqEk9CM0I2hFX3oXa5T2oU86OXgMSW4s6U="
ul.MONGO_URI = "mongodb://AdminUpSwingGlobal:Upswing098812Admin0165r@dev.db.upswing.global:27017/?authSource=admin&readPreference=primary&appname=Agent%20Oracle%20Dev&ssl=false"
ul.G_CLOUD_PROJECT = "aura-staging-31cae"
ul.FIREBASE = "SECRET/aura-staging-31cae-firebase-adminsdk-dyolr-7c135838e9.json"
ul.LOG_LEVEL_VALUE = 'DEBUG'


try:
    firebase_admin.get_app()
except Exception as e:
    logger.info(e)
    logger.info('Initializing default firebase app')
    cred = firebase_admin.credentials.Certificate(
        os.getenv("FIREBASE", "/Users/harsh/upswing/github/api-oracle/SECRET/aura-staging-31cae-firebase-adminsdk-dyolr-7c135838e9.json"))
    firebase = firebase_admin.initialize_app(cred)


if __name__ == '__main__':
    print('testing guest add')
    guest = GuestsModel(_id='12345')
    guest.firstName = 'harsh'
    guest.lastName = 'mathur'
    mongo = MongodbV2('OHIBP2')
    mongo.save(guest)
    mongo.close_connection()
    print('completed')



from .property import Property, PropertyAirport, PropertyTransportation, PropertyCreditCardTypes, PropertyHotelAmenities, \
    PropertyAccountsReceivableDetails
from .auth import Token
from .alvie import AlvieRating, Alvie
from .pubSub import ReservationMsg
from .reservation import Reservation
from .country import Country
from .organization_cfg import CommunicationMethodsEntDetails,HotelTaxType, LeisureActivityTypes
from .http import ResponseList, ResponseDict



from setuptools import setup, find_packages

VERSION = '2.2.32'
DESCRIPTION = 'Utility package'
LONG_DESCRIPTION = 'Utility package for upswing applications'

# Setting up
setup(
    name="upswingutil",
    version=VERSION,
    author="Harsh Mathur",
    author_email="hmathur@upswing.cloud",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=[
        'requests',
        'pydantic',
        'gcsfs',
        'cryptography',
        'fastapi',
        'firebase_admin',
        'pymongo',
        'loguru',
        'mongoengine',
        'pycountry-convert',
        'google-cloud-secret-manager==2.0.0',
        'google-api-python-client'
    ],
    keywords=['python', 'upswing'],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Microsoft :: Windows",
    ]
)



from .cloud import get_model_from_cloud_storage, upload_model_to_cloud_storage
from .crypto import encrypt, decrypt
from .http_retry import http_retry
from .jwt import verify_and_decode_jwt, verify_owner_and_get_id
from .logger import setup_logging
from .secret_manager import access_secret_version, add_secret_version
from .dataflow_job import create_dataflow_job_for_sending_emails


import unittest
from upswingutil.ml import Classification_Cancellation_NoShow_Others
import upswingutil as ul
from upswingutil.pms import PMS

ul.ENCRYPTION_SECRET = "S1335HwpKYqEk9CM0I2hFX3oXa5T2oU86OXgMSW4s6U="
ul.MONGO_URI = "mongodb://AdminUpSwingGlobal:Upswing098812Admin0165r@dev.db.upswing.global:27017/?authSource=admin&readPreference=primary&appname=Agent%20Oracle%20Dev&ssl=false"
ul.G_CLOUD_PROJECT = "aura-staging-31cae"
ul.FIREBASE = "/Users/harsh/upswing/github/api-oracle/SECRET/aura-staging-31cae-firebase-adminsdk-dyolr-7c135838e9.json"
ul.LOG_LEVEL_VALUE = 'DEBUG'

class TestClassification_Cancellation_NoShow_Others(unittest.TestCase):

    def test_creating_model_oracle(self):
        """
        Training model for ORACLE
        """
        orgId = 'OHIPSB'
        classifier_cancel_noshow_others = Classification_Cancellation_NoShow_Others(PMS.ORACLE, orgId)
        # classifier_cancel_noshow_others.preprocess()
        # classifier_cancel_noshow_others.train()

    def test_creating_model_rms(self):
        """
        Training model for RMS
        """
        orgId = '11249'
        classifier_cancel_noshow_others = Classification_Cancellation_NoShow_Others(PMS.RMS, orgId)
        classifier_cancel_noshow_others.preprocess()
        classifier_cancel_noshow_others.train()

    def test_predict_rms(self):
        '''
        Creating the Model and Make Predictions on the Trained Model
        '''
        orgId = '11249'
        classifier_cancel_noshow_others = Classification_Cancellation_NoShow_Others(PMS.RMS, orgId)
        ## Features to Predict on:
        clientId= 0                             ## Int
        rateTypeName='Old system rate'          ## String
        Class= 'None'                           ## String
        durationOfStay= 11.0                    ## Float
        bookingWindowDays= 52.0                 ## Float
        guestCount= 1                           ## Int

        ## For Checking the results
        result = "Cancelled"

        final, overall = classifier_cancel_noshow_others.predict(clientId, rateTypeName, Class, durationOfStay, bookingWindowDays, guestCount)
        assert result==final


    def test_predict_oracle(self):
        pass


if __name__ == '__main__':
    unittest.main()


import logging
import upswingutil as ul
from firebase_admin import firestore, get_app, storage


class Firestore:
    org_collection = 'Organizations'
    sub_collection_properties = 'properties'
    credentials = 'credentials'

    def __init__(self, app=None, project_name = ul.G_CLOUD_PROJECT):
        self.firestore_db = firestore.client(app=get_app(app)) if app else firestore.client()
        self.firestore_storage = storage.bucket(name=f'{project_name}.appspot.com', app=get_app(app)) if app else storage.bucket(name=f'{project_name}.appspot.com')

    def get_collection(self, name):
        return self.firestore_db.collection(name)

    def get_collection_docs(self, name):
        return self.firestore_db.collection(name).stream()

    def get_ref_collection(self, name):
        return self.firestore_db.collection(name)

    def get_ref_document(self, name, doc):
        return self.firestore_db.collection(name).document(doc).get()

    def write_doc(self, collection: str, doc: str, data: dict):
        __ref__ = self.firestore_db.collection(collection).document(doc)
        __ref__.set(data, merge=True)
        logging.debug(f"Document written successfully")

    def update_doc(self, collection: str, doc: str, data: dict):
        __ref__ = self.firestore_db.collection(collection).document(doc)
        __ref__.update(data)
        logging.debug(f"Document Updated successfully")

    def increment_doc_value(self, collection: str, doc: str, data: str, incr: int):
        __ref__ = self.firestore_db.collection(collection).document(doc)
        __ref__.update({
            data: firestore.firestore.Increment(incr)
        })

    def upload_document_firestorage(self, path: str, contentType: str,imgData: bytes, public: bool = False):
        blob_ref = self.firestore_storage.blob(path)
        blob_ref.upload_from_string(imgData, content_type=contentType)
        if public:
            blob_ref.make_public()
        return blob_ref.public_url

    def delete_document_firestorage(self, path: str):
        blob_ref = self.firestore_storage.blob(path)
        blob_ref.delete()
        return True



from googleapiclient.discovery import build
import upswingutil as ul

def create_dataflow_job_for_sending_emails(jobname: str, parameters: dict):
    dataflow = build('dataflow', 'v1b3')
    request = dataflow.projects().locations().templates().launch(
        projectId=ul.G_CLOUD_PROJECT,
        location='asia-south1',
        gcsPath="gs://dataflow-content/Communication/EmailTrigger/templates/email_trigger",
        body={
            'jobName': jobname,
            'parameters': parameters
        }
    )
    response = request.execute()
    return response


import os
import logging
import sys
from loguru import logger
import upswingutil as ul


LOG_LEVEL = logging.getLevelName(os.environ.get("LOG_LEVEL", ul.LOG_LEVEL_VALUE))
JSON_LOGS = True if os.environ.get("JSON_LOGS", "0") == "1" else False


class InterceptHandler(logging.Handler):
    def emit(self, record):
        # Get corresponding Loguru level if it exists
        try:
            level = logger.level(record.levelname).name
        except ValueError:
            level = record.levelno

        # Find caller from where originated the logged message
        frame, depth = logging.currentframe(), 2
        while frame.f_code.co_filename == logging.__file__:
            frame = frame.f_back
            depth += 1

        logger.opt(depth=depth, exception=record.exc_info).log(level, record.getMessage())


def setup_logging():
    # intercept everything at the root logger
    logging.root.handlers = [InterceptHandler()]
    logging.root.setLevel(LOG_LEVEL)

    # remove every other logger's handlers
    # and propagate to root logger
    for name in logging.root.manager.loggerDict.keys():
        logging.getLogger(name).handlers = []
        logging.getLogger(name).propagate = True

    # configure loguru
    logger.configure(handlers=[{"sink": sys.stdout, "serialize": JSON_LOGS}])
